import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import gl4java.awt.GLCanvas;
import gl4java.utils.glut.*;
import gl4java.utils.textures.*;

/**
* Dette er hovedklassen, det er denne klassen som tegner ut
* ut monsteret.
*/

public class Monster extends JFrame implements ActionListener {
  	
  /*Variabler som har med vinduet og gjre*/	
  JPanel contentPane;
  MouseHandler mHandler;
  WindowHandler wHandler;
  myGLCanvas canvas_c = null;
  
  /*Instanser av de andre klassene*/
  MyColor myColor;
  Bein bein = new Bein();
  Hand hand = new Hand();
  Head head = new Head();
  Horn horn = new Horn();
  Munn munn = new Munn();
  Leggen leggen = new Leggen();
  Klo klo = new Klo();
  FotKlo fotKlo = new FotKlo();
  KloInn kloInn = new KloInn();
  HornInn hornInn = new HornInn();
  Arm arm = new Arm();
  UnderArm underArm = new UnderArm();
  Eye eye = new Eye();
  Eyelid eyelid = new Eyelid();
  
  /*Variabler som har med musbevegelser  gjre*/
  boolean m_is_moving=false;
  float lastx=0.0f;
  float lastz=0.0f;
  float lasty=0.0f;
  float rotx=0.0f;
  float rotz=0.0f;
  float roty=0.0f;

  /*Array med koordinatene til teksturen
  *Her sier jeg at teksturen skal legges en gang
  *over hele flaten.
  */  
  float textPoints[]={
            0.0f, 0.0f,
            1.0f, 0.0f,
            0.0f, 1.0f,
            1.0f, 1.1f,
  };
   

  public Monster() {

    /** Setter opp vinduet */
    contentPane = (JPanel)this.getContentPane();
    contentPane.setLayout(new BorderLayout());
    this.setSize(new Dimension(500, 500));
    this.setTitle("Monster");
    
    /** Menyen */
    MenuBar menybar = new MenuBar();
    this.setMenuBar(menybar);
    Menu meny = new Menu("Monster");
     
    menybar.add(meny);
         
    MenuItem snu = new MenuItem("Rundtur");
    meny.add(snu);
	 snu.addActionListener(this);
	 
    meny.addSeparator();

    MenuItem exit = new MenuItem("Exit");
    meny.add(exit);
    exit.addActionListener(this);
    
    /** Gjr klart for gl */
    Dimension dimension = getSize();
    canvas_c = new myGLCanvas(dimension.width, dimension.height);
    contentPane.add("Center", canvas_c);

    wHandler=new WindowHandler();
    this.addWindowListener(wHandler);

    mHandler = new MouseHandler();
    canvas_c.addMouseListener( mHandler );
    canvas_c.addMouseMotionListener( mHandler );

    /** Gjr vinduet synlig */
    this.setVisible(true);

  }

  /** Starter applikasjonen */
  public static void main(String args[]) {

    new Monster();

  }

	/** Metode som tar hnd om brukerens handlinger */
  public void actionPerformed(ActionEvent e) {
	
		if(e.getActionCommand()== "Rundtur") {
			canvas_c.turn();
		}
		if(e.getActionCommand()== "Exit") {
			System.exit(0);
		}
	}
   
   /** Klasse som tar seg av vinduet */
  	class WindowHandler implements WindowListener {

     public void windowOpened(WindowEvent parm1) {}
     public void windowClosing(WindowEvent parm1) {
      	System.exit( 0 );
    	}
    public void windowClosed(WindowEvent parm1) {}
    public void windowIconified(WindowEvent parm1) {}
    public void windowDeiconified(WindowEvent parm1) {}
    public void windowActivated(WindowEvent parm1) {}
    public void windowDeactivated(WindowEvent parm1) {}

  }

  /** Tar imot museklikk - bevegelser */
  class MouseHandler implements MouseListener, MouseMotionListener {

    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {
        m_is_moving=false;
        canvas_c.repaint();
    }

    public void mousePressed(MouseEvent e) {

      lastx = e.getX();
      lastz = e.getY();
      m_is_moving=true;
    }

    public void mouseDragged(MouseEvent e) {

      if(!m_is_moving)
        return;

      int x = e.getX();
      int y = e.getY();
      Dimension dim = e.getComponent().getSize();
      int width = dim.width;
      int height = dim.height;

      rotz += (90.0f*((float)(x-lastx)/(float)(width)));
      rotx += (90.0f*((float)(y-lastz)/(float)(height)));

      lastx = x;
      lastz = y;

      canvas_c.repaint();
    }
  }

   
  /**
  *GL-klassen, tegner ut monsteret
  */
  private class myGLCanvas extends GLCanvas {

    GLUTFunc glut = null;
    int []texName = new int[1];  //Array for teksturen
    int fill_line = gl.GL_FILL;  //GL_FILL eller GL_LINES
        	
    public myGLCanvas(int width, int height) {
      super(width, height);
    }

	 /** 
	 *Setter noen variabler som har med utegning  gjre
    */
    public void preInit() {
    	
      doubleBuffer = true; 
      stereoView = false;
      accumSize = 8;
    }

    /**
    *Gjr klar lyssetting, bakgrunnsfarge osv.
    */
    public void init() {
      
      glut = new GLUTFuncLightImpl(gl, glu);
      myColor = new MyColor(getSize().width, getSize().height);
      reshape(getSize().width, getSize().height);

      /** Lyskilde */
      float ambient[] = {0.4f,0.4f,0.4f,1.0f };
      float diffuse[] = {1.0f,1.0f,1.0f,1.0f };
      float position[] = {1.0f,1.0f,1.0f,0.0f };
      float lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };

      gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
      gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
      gl.glLightfv(GL_LIGHT0, GL_POSITION, position);

      gl.glEnable(GL_LIGHT0);
      gl.glEnable(GL_LIGHTING);

      /** smooth */
      gl.glShadeModel(GL_SMOOTH);

      /** depth sorting */
      gl.glEnable(GL_DEPTH_TEST);
      gl.glDepthFunc(GL_LESS);

		/** Svart bakgrunnsfarge */
      gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

    }
    
    /**
    *Displaymetoden, plasserer yet og kaller p tegnemetoden
    */
    public void display() {

      if (glj.gljMakeCurrent() == false)
        return;

      gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      gl.glClear(GL_ACCUM_BUFFER_BIT);
      gl.glLoadIdentity();

      glu.gluLookAt(  0.0f,30.0f,5.0f, // ye, avstand settes
                      0.0f,0.0f,0.0f,  // Ser p
                      0.0f,0.0f,1.0f);  // Velger hvilken som skal vre opp

      /** Her kan jeg flytte p objektet, jeg lar det st midt i */
      gl.glTranslatef(0.0f,0.0f,0.0f);

      // x-aksen
      gl.glRotatef(rotx,1.0f,0.0f,0.0f);
      // y-aksen
      gl.glRotatef(roty,0.0f,0.0f,1.0f);
      // z-aksen
      gl.glRotatef(rotz,0.0f,1.0f,0.0f);

		/** Metoden der jeg kaller tegnemetodene */
      makeMonster();
		
		/** "Rydder" opp etter meg */
      gl.glFlush();

      glj.gljSwap();
      glj.gljFree();

   }

   /**
   *Metode som snur monsteret 360' rundt
   */
   
   public void turn() {
    for(float i=0.0f; i<=360.0f; i+=40.0f) {
        roty = i;
        display();
    }
   }
   
   /**
   *Metode som kaller p alle tegnemetodene
   */
    private void makeMonster() {
		
      gl.glEnable(GL_TEXTURE_2D); //Skal bruke teksturer, s vi enabler de     
      drawHead();
      gl.glDisable(GL_TEXTURE_2D); //Disabler teksturene igjen
      drawHornLeft();
      drawHornRight();
      drawMouth();
      gl.glEnable(GL_TEXTURE_2D);
      drawLeftArm();
      gl.glDisable(GL_TEXTURE_2D);
      gl.glEnable(GL_TEXTURE_2D);
      drawRightArm();
      gl.glDisable(GL_TEXTURE_2D);
      gl.glEnable(GL_TEXTURE_2D);
      drawLeftLeg();
      gl.glDisable(GL_TEXTURE_2D);
      gl.glEnable(GL_TEXTURE_2D);
      drawRightLeg();
		gl.glDisable(GL_TEXTURE_2D);


      gl.glEnable(GL_TEXTURE_2D);      
      drawEyelid();
      gl.glDisable(GL_TEXTURE_2D);

      
      gl.glEnable(GL_TEXTURE_2D);      
      drawEye();
      gl.glDisable(GL_TEXTURE_2D);
     
      
	 }
	 
	 /**
	 *Blir kallt hvis GLCanvas.componentResize  har blitt kallt.
	 *w og h er den nye bredden og den nye hyden p canvaset.
	 */
    public void reshape(int w, int h) {

      gl.glViewport(0, 0, w, h);
      gl.glMatrixMode(GL_PROJECTION);
      gl.glLoadIdentity();
      glu.gluPerspective( 45.0, (float) w/h, 1.0, 500.0 );
      gl.glMatrixMode(GL_MODELVIEW);

    }

    /**
    *Tegner hodet/kroppen p monsteret og laster inn teksturer
    */

    public void drawHead() {

      String fileHeadTexture = "skin256.png";	//Bildet som skal brukes
      textures(fileHeadTexture);	//Kaller p teksturmetoden min
      
      gl.glTranslatef(0.0f,0.0f,-5.0f);
      myColor.monsterGreen();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);
      
      /** Fikser normalene for meg :) */
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
      
      /** Hvordan teksturen skal legges ut */
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
      gl.glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		/** Gjr klar bezierflaten */
      gl.glMap2f(GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,
              head.getUN(), //Fr de ndvendige punktene fra hodeklassen
              0.0f,1.0f,3*head.getUN(),head.getVN(),head.getListe());
      
      /** Gjr klar til  legge ut teksturen */         
      gl.glMap2f(GL_MAP2_TEXTURE_COORD_2,
              0,1,2,2,
              0,1,4,2,textPoints);  
       
      gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      gl.glEnable(GL_MAP2_VERTEX_3);
      
      /** Tegner ut */  
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f); 
      gl.glFrontFace(gl.GL_CW);          
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      /** Tegner ut andre halvdel av hodet */
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      gl.glFrontFace(gl.GL_CW);
        
 
  }

  /**
  *Tegner ut det venstre hornet p monsteret
  */
  public void drawHornLeft() {

      gl.glPushMatrix();
      /** Flytter og roterer det dit jeg vil ha det */
      gl.glTranslatef(-1.5f,0.0f,8.5f);
      gl.glRotatef(90.0f,0.0f,0.0f,1.0f);
      gl.glRotatef(-20.0f,1.0f,0.0f,0.0f);
      
      /** Farge og lys */
      myColor.whiteBrown();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      /** Horn ytterkant */
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,horn.getUN(),
              0.0f,1.0f,3*horn.getUN(),horn.getVN(),horn.getListe());

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
 
      /** Innerside horn har en litt annen form */
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
 
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hornInn.getUN(),
              0.0f,1.0f,3*hornInn.getUN(),hornInn.getVN(),hornInn.getListe());

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

      gl.glPopMatrix();


  }

  /**
  * Hyre horn! Samme som venstre, bare annen plassering
  */
  public void drawHornRight() {

      gl.glPushMatrix();
      gl.glTranslatef(1.5f,0.0f,8.5f);
      gl.glRotatef(-90.0f,0.0f,0.0f,1.0f);
      gl.glRotatef(-20.0f,1.0f,0.0f,0.0f);
      
      myColor.whiteBrown();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

		/** Horn ytterkant */
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,horn.getUN(),
              0.0f,1.0f,3*horn.getUN(),horn.getVN(),horn.getListe());

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
  
      
      /** Innerside horn */
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
 
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hornInn.getUN(),
              0.0f,1.0f,3*hornInn.getUN(),hornInn.getVN(),hornInn.getListe());

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

      gl.glPopMatrix();
          
  }

   /**
   * Venstre arm, skal ha annen tekstur, derfor m den lastes inn
   */
   public void drawLeftArm() {

		/** Teksturkode */
      String fileHeadTexture = "skin64.png";
      textures(fileHeadTexture);

      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,0);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
      
      
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
      gl.glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
              
      gl.glMap2f(GL_MAP2_TEXTURE_COORD_2,
              0,1,2,2,
              0,1,4,2,textPoints);  
      
      /** tegnekode */        
      gl.glPushMatrix();
      
      /** Overarmen */
      gl.glTranslatef(-4.8f,-1.0f,2.1f);
      gl.glRotatef(8.0f,0.0f,1.0f,0.0f);
      
      myColor.monsterGreen();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,arm.getUN(),
              0.0f,1.0f,3*arm.getUN(),arm.getVN(),arm.getListe());

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);


      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      /** Underarmen */
      gl.glPushMatrix();
      
      /** Flytter ned s den begynner der overarmen slutter */
      gl.glTranslatef(0.0f,0.0f,-3.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,underArm.getUN(),
              0.0f,1.0f,3*underArm.getUN(),underArm.getVN(),underArm.getListe());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

      /** Hndflata */

      gl.glPushMatrix();

      gl.glTranslatef(-0.2f,-0.5f,-0.5f);
		gl.glRotatef(-25.0f,0.0f,0.0f,1.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hand.getUN(),
              0.0f,1.0f,3*hand.getUN(),hand.getVN(),hand.getListeAndre());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
		
		/** Andre siden av hndflata */
		gl.glPushMatrix();
		gl.glTranslatef(0.0f,1.0f,0.0f);
		
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hand.getUN(),
              0.0f,1.0f,3*hand.getUN(),hand.getVN(),hand.getListe());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
		
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);      
    	gl.glPopMatrix();
 
   	
    	/** Fingre */
		float tx=-0.35f;
		float ty=-1.1f;
		float tz=-0.5f;
		float degree = -10.0f;

		/* Lkke som legger ut fingrene */
		for(int i=0; i<=2; i++) {

			if(i>0)
				gl.glEnable(GL_TEXTURE_2D);

			gl.glPushMatrix();
	    	
			gl.glRotatef(90.0f,1.0f,0.0f,0.0f);
			gl.glRotatef(degree,0.0f,0.0f,1.0f);
			gl.glTranslatef(tx,ty,tz);
					
      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hand.getUN(),
              0.0f,1.0f,3*hand.getUN(),hand.getVNfinger(),hand.getFinger());
      
      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);
		
      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      	      
      	gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
     		gl.glEvalMesh2(fill_line,0,40,0,40);
     		
     		/** For  fylle fingrene */
     		gl.glPushMatrix();
     		myColor.monsterGreen();		     		
     		glut.glutSolidSphere(0.15f,20,20);
			gl.glPopMatrix(); 
     		
     		gl.glDisable(GL_TEXTURE_2D); //Disabler her for klrne skal ikke ha tekstur
     		/** Klo */
     		gl.glPushMatrix();
     		myColor.whiteBrown();
     		gl.glTranslatef(0.0f,-0.2f,0.0f);		
     		gl.glRotatef(90.0f,1.0f,0.0f,0.0f);
     		gl.glRotatef(180.0f,0.0f,0.0f,1.0f);

      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,klo.getUN(),
              0.0f,1.0f,3*klo.getUN(),klo.getVN(),klo.getListe());
      
      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);
		
      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      	      
      	gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
     		gl.glEvalMesh2(fill_line,0,40,0,40);
     		
			gl.glPopMatrix();
     		     		
     		gl.glPopMatrix();
     		
     		tx = tx + 0.4f;
     		degree = degree + 5.0f;
    	}	


		/** Tommelen lager jeg for seg selv */
		gl.glPushMatrix();
		gl.glEnable(GL_TEXTURE_2D);
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
      	0.0f,1.0f,3,hand.getUN(),
         0.0f,1.0f,3*hand.getUN(),hand.getVNfinger(),hand.getFinger());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
		
		
		gl.glTranslatef(1.0f,0.6f,-0.8f);
		gl.glRotatef(90.0f,1.0f,0.0f,0.0f);
		gl.glRotatef(30.0f,0.0f,0.0f,1.0f);
		gl.glEvalMesh2(GL_FILL,0,40,0,40);
     	gl.glPopMatrix(); //Tommel
     	gl.glDisable(GL_TEXTURE_2D);
     	
     	//Tommelklo
     	gl.glPushMatrix();
     	myColor.whiteBrown();
     	gl.glTranslatef(1.0f,0.6f,-0.8f);		
     	gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
     	gl.glRotatef(30.0f,0.0f,1.0f,0.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,klo.getUN(),
              0.0f,1.0f,3*klo.getUN(),klo.getVN(),klo.getListe());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
		
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40); 
           
      gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
     	gl.glEvalMesh2(fill_line,0,40,0,40);
     		
		gl.glPopMatrix(); //Tommelklo

    	
    	
    	gl.glPopMatrix(); //handa
      gl.glPopMatrix(); //underarmen
      gl.glPopMatrix(); //Overarmen


    }

   /**
   * Hyre arm
   */
    public void drawRightArm() {

      gl.glPushMatrix();
      gl.glTranslatef(4.8f,-1.0f,2.1f);
      gl.glRotatef(-8.0f,0.0f,1.0f,0.0f);
      
      myColor.monsterGreen();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);
		
		/** Overarmen */
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,arm.getUN(),
              0.0f,1.0f,3*arm.getUN(),arm.getVN(),arm.getListe());

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);


      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      /** Underarmen */
      gl.glPushMatrix();
      gl.glTranslatef(0.0f,0.0f,-3.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,underArm.getUN(),
              0.0f,1.0f,3*underArm.getUN(),underArm.getVN(),underArm.getListe());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);


      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      
      /** Handa */

      gl.glPushMatrix();
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glRotatef(25.0f,0.0f,0.0f,1.0f);
      gl.glTranslatef(0.0f,0.5f,-0.5f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hand.getUN(),
              0.0f,1.0f,3*hand.getUN(),hand.getVN(),hand.getListe());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glPushMatrix();
      
		//Andre siden av hndflata
		gl.glTranslatef(0.0f,-1.0f,0.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hand.getUN(),
              0.0f,1.0f,3*hand.getUN(),hand.getVN(),hand.getListeAndre());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40); 
      gl.glPopMatrix();

      /** Fingre */
		float tx=-0.35f;
		float ty=-1.1f;
		float tz=0.5f;
		float degree = -10.0f;

		for(int i=0; i<=2; i++) {

			if(i>0)
				gl.glEnable(GL_TEXTURE_2D);

			gl.glPushMatrix();
	    	
			gl.glRotatef(90.0f,1.0f,0.0f,0.0f);
			gl.glRotatef(degree,0.0f,0.0f,1.0f);
			gl.glTranslatef(tx,ty,tz);		
      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,hand.getUN(),
              0.0f,1.0f,3*hand.getUN(),hand.getVNfinger(),hand.getFinger());
      
      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);
		
      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);      
      	gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
     		gl.glEvalMesh2(fill_line,0,40,0,40);
     		
     		//For  fylle fingrene
     		gl.glPushMatrix(); 
     		myColor.monsterGreen();		     		
     		glut.glutSolidSphere(0.15f,20,20);
			gl.glPopMatrix(); 
     		
     		gl.glDisable(GL_TEXTURE_2D);
     		
     		//Klo
     		gl.glPushMatrix();
     		myColor.whiteBrown();
     		gl.glTranslatef(0.0f,-0.2f,0.0f);		
     		gl.glRotatef(90.0f,1.0f,0.0f,0.0f);
     		
      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,klo.getUN(),
              0.0f,1.0f,3*klo.getUN(),klo.getVN(),klo.getListe());
      
      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);
		
      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      	      
      	gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
     		gl.glEvalMesh2(fill_line,0,40,0,40);
     		
			gl.glPopMatrix();
     		     		
     		gl.glPopMatrix(); //Finger
     		tx = tx + 0.4f;
     		degree = degree + 5.0f;
    	}	

		/** Tommel */
		gl.glPushMatrix();
		gl.glEnable(GL_TEXTURE_2D);
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
      	0.0f,1.0f,3,hand.getUN(),
         0.0f,1.0f,3*hand.getUN(),hand.getVNfinger(),hand.getFinger());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
		
		
		gl.glTranslatef(1.0f,-0.6f,-0.8f);
		gl.glRotatef(90.0f,1.0f,0.0f,0.0f);
		gl.glRotatef(30.0f,0.0f,0.0f,1.0f);
		
		gl.glEvalMesh2(GL_FILL,0,40,0,40);
     	gl.glPopMatrix(); //Tommel
     	gl.glDisable(GL_TEXTURE_2D);
     	
     	//Tommelklo
     	gl.glPushMatrix();
     	myColor.whiteBrown();
     	gl.glTranslatef(1.0f,-0.6f,-0.8f);		
     	gl.glRotatef(-30.0f,0.0f,1.0f,0.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,klo.getUN(),
              0.0f,1.0f,3*klo.getUN(),klo.getVN(),klo.getListe());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);
		
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);      
      gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
     	gl.glEvalMesh2(fill_line,0,40,0,40);
     		
		gl.glPopMatrix(); //Tommelklo      
                 
    
    	gl.glPopMatrix(); //handa
      gl.glPopMatrix(); //underarmen
      gl.glPopMatrix(); //Overarmen


    }

   /**
   * Venstre ben
   */
    public void drawLeftLeg() {

      gl.glPushMatrix();
      
      /** Lr */
      gl.glTranslatef(-2.5f,0.0f,-1.5f);
      gl.glRotatef(7.0f,0.0f,1.0f,0.0f);
      myColor.monsterGreen();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,bein.getUN(),
              0.0f,1.0f,3*bein.getUN(),bein.getVN(),bein.getListe());

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);


      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      /** leggen */
      gl.glPushMatrix();
      glut.glutSolidSphere(0.25f,20,20);
      
      gl.glTranslatef(-0.4f,0.0f,-3.0f);
      gl.glRotatef(7.0f,0.0f,1.0f,0.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,leggen.getUN(),
              0.0f,1.0f,3*leggen.getUN(),leggen.getVN(),leggen.getListeLeggen());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);


      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
     	
     		
     	gl.glPushMatrix();
     	gl.glTranslatef(0.0f,0.0f,0.15f);
     	myColor.monsterGreen();		     		
     	glut.glutSolidSphere(0.25f,20,20);
		gl.glPopMatrix(); 
    	
    	/** Foten */     
      
      float degree = -45.0f;
      float x=-0.05f;
      float y=-2.0f;
      float z=0.3f;
      
		/** Lkke som lager tre tr med klr */
		for(int i=0; i<=2; i++) {

      	gl.glPushMatrix();
      	gl.glRotatef(degree,0.0f,0.0f,1.0f);
      	gl.glTranslatef(x,y,z);

     		gl.glPushMatrix(); 
     		gl.glTranslatef(0.0f,1.7f,0.0f);
     		myColor.monsterGreen();		     		
     		glut.glutSolidSphere(0.25f,20,20);
			gl.glPopMatrix(); 

      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,leggen.getUNfot(),
              0.0f,1.0f,3*leggen.getUNfot(),leggen.getVNfot(),leggen.getFot());
      
      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);
			
      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      	gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      	
      	//Klr
			gl.glPushMatrix();
      	
      	gl.glDisable(GL_TEXTURE_2D);
      	myColor.whiteBrown();
      	gl.glRotatef(90,1.0f,0.0f,0.0f);
      	gl.glRotatef(180,0.0f,0.0f,1.0f);
      	gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

     	 	//Klr ytterkant
      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,fotKlo.getUN(),
              0.0f,1.0f,3*fotKlo.getUN(),fotKlo.getVN(),fotKlo.getListe());

      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);


      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
 
      	//Innerside klr.. :)
      	gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
 
      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,fotKlo.getUN(),
              0.0f,1.0f,3*fotKlo.getUN(),fotKlo.getVN(),fotKlo.getListeInn());

      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

			gl.glEnable(GL_TEXTURE_2D);
      	gl.glPopMatrix();//Klr
    	
    		gl.glPopMatrix();//"Tr"
    		
    		x = x + 0.07f;
    		degree = degree + 25.0f;
    	}
    	
      gl.glPopMatrix(); //lret
      gl.glPopMatrix(); //leggen


   }

   /**
   * Venstre ben
   */
    public void drawRightLeg() {

      gl.glPushMatrix();
      /** Lret */
      gl.glTranslatef(2.5f,0.0f,-1.5f);
      gl.glRotatef(-7.0f,0.0f,1.0f,0.0f);
      
      myColor.monsterGreen();
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,bein.getUN(),
              0.0f,1.0f,3*bein.getUN(),bein.getVN(),bein.getListe());

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);


      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      /** Leggen */
      gl.glPushMatrix();
      glut.glutSolidSphere(0.25f,20,20);
      
      gl.glTranslatef(0.4f,0.0f,-3.0f);
      gl.glRotatef(-7.0f,0.0f,1.0f,0.0f);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,leggen.getUN(),
              0.0f,1.0f,3*leggen.getUN(),leggen.getVN(),leggen.getListeLeggen());
      
      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

     	gl.glPushMatrix(); //Fyllekule
     	gl.glTranslatef(0.0f,0.0f,0.15f);
     	myColor.monsterGreen();		     		
     	glut.glutSolidSphere(0.25f,20,20);
		gl.glPopMatrix(); //Fyllekule

    	/** Foten */      
      float degree = -5.0f;
      float x=-0.05f;
      float y=-2.0f;
      float z=0.3f;

		for(int i=0; i<=2; i++) {
      	gl.glPushMatrix();
      	gl.glRotatef(degree,0.0f,0.0f,1.0f);
      	gl.glTranslatef(x,y,z);

      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,leggen.getUNfot(),
              0.0f,1.0f,3*leggen.getUNfot(),leggen.getVNfot(),leggen.getFot());
      
      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);


      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
      
      	gl.glRotatef(180.0f,0.0f,1.0f,0.0f);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
    		
    		/** Klo */		
      	gl.glPushMatrix();
      	gl.glDisable(GL_TEXTURE_2D);
      	myColor.whiteBrown();
      	gl.glRotatef(90,1.0f,0.0f,0.0f);
      	gl.glRotatef(180,0.0f,0.0f,1.0f);
      	gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,fotKlo.getUN(),
              0.0f,1.0f,3*fotKlo.getUN(),fotKlo.getVN(),fotKlo.getListe());

      	gl.glEnable(gl.GL_MAP2_VERTEX_3);
      	gl.glEnable(gl.GL_AUTO_NORMAL);
      	gl.glEnable(gl.GL_NORMALIZE);

      	gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      	gl.glFrontFace(gl.GL_CW);
      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
 
      	//Innerside klo
      	gl.glRotatef(180.0f,0.0f,0.0f,1.0f);
 
      	gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,fotKlo.getUN(),
              0.0f,1.0f,3*fotKlo.getUN(),fotKlo.getVN(),fotKlo.getListeInn());

      	gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

			gl.glEnable(GL_TEXTURE_2D);
      	gl.glPopMatrix();

    		gl.glPopMatrix();
    		
    		x = x + 0.07f;
    		degree = degree + 20.0f;
    	}    
    	
      gl.glPopMatrix(); //lret
      gl.glPopMatrix(); //leggen

   }
   
   /**
   * yelokk og litt ekstra hud under yet :)
   */
   public void drawEyelid() {

      gl.glPushMatrix();
      
      /** Litt hud under yet */
      gl.glTranslatef(0.0f,-4.15f,6.3f);
      gl.glRotatef(-50.0f,1.0f,0.0f,0.0f);

      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,eyelid.getUN(),
              0.0f,1.0f,3*eyelid.getUN(),eyelid.getVN(),eyelid.getListe());

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
  
      gl.glPopMatrix();   
      
      /** yelokk */
      gl.glPushMatrix();

      gl.glTranslatef(0.0f,-2.0f,9.2f);
      gl.glRotatef(155.0f,1.0f,0.0f,0.0f);

      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,eyelid.getUN(),
              0.0f,1.0f,3*eyelid.getUN(),eyelid.getVNto(),eyelid.getListeTo());

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);
      gl.glFrontFace(gl.GL_CW);
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

      gl.glPopMatrix();      	
   } 
    
   /**
   * yet med tekstur
   */
   public void drawEye() {

		//Laster inn ny tekstur
      textures("eye_green.png");
      
      gl.glPushMatrix();      
      gl.glTranslatef(0.0f,-3.2f,7.5f);
      gl.glRotatef(-33.0f,1.0f,0.0f,0.0f);

      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
      gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
      gl.glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
        
      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,eye.getUN(),
              0.0f,1.0f,3*eye.getUN(),eye.getVN(),eye.getListe());
               
      gl.glMap2f(GL_MAP2_TEXTURE_COORD_2,
              0,1,2,2,
              0,1,4,2,textPoints);  
       
      gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      gl.glEnable(GL_MAP2_VERTEX_3);
      
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f); 
      gl.glFrontFace(gl.GL_CW);         
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);
 
 		gl.glPopMatrix();	


	}

   /**
   * Munnen..
   */
    public void drawMouth() {

		myColor.blackBrown();	//Ingen tekstur, bare farge

      gl.glPushMatrix();      
      gl.glTranslatef(0.0f,-4.6f,3.8f);

      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,munn.getUN(),
              0.0f,1.0f,3*munn.getUN(),munn.getVN(),munn.getListe());
               
      gl.glEnable(GL_MAP2_VERTEX_3);
      
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f); 
      gl.glFrontFace(gl.GL_CW);         
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

		/** Tennene */
		gl.glPushMatrix();
 	
 		myColor.whiteTeeth();
 		
      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
      	0.0f,1.0f,3,munn.getUN(),
         0.0f,1.0f,3*munn.getUN(),munn.getVN(),munn.getListeTann());
               
      gl.glEnable(GL_MAP2_VERTEX_3);
      
      /** Frste tann */
  		gl.glTranslatef(-1.1f,0.0f,-0.1f);
	   gl.glRotatef(33.0f,0.0f,1.0f,0.0f);    	

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);          
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

  		/** Andre tann */
  		gl.glTranslatef(0.75f,0.0f,0.2f);
	   gl.glRotatef(-25.0f,0.0f,1.0f,0.0f);    	

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f);          
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

  		/** Tredje tann */
  		gl.glTranslatef(0.75f,0.0f,0.12f);
	   gl.glRotatef(-16.0f,0.0f,1.0f,0.0f);    	

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f,1.0f);          
      gl.glEvalMesh2(fill_line,0,40,0,40);

      /** Fjerde tann */
      gl.glTranslatef(0.75f,0.0f,0.15f);
      gl.glRotatef(-24.0f,0.0f,1.0f,0.0f);    	

      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f,1.0f);          
      gl.glEvalMesh2(fill_line,0,40,0,40);
      
      /** "Leppa" For  f penere tenner */
      gl.glPushMatrix();  
      
      gl.glEnable(GL_TEXTURE_2D);	//Her skal jeg ha p tekstur igjen  
      gl.glTranslatef(-1.1f,0.15f,0.35f);
      gl.glRotatef(32.0f,0.0f,1.0f,0.0f);    	

      gl.glLightModeli(gl.GL_LIGHT_MODEL_TWO_SIDE,1);

      gl.glEnable(gl.GL_MAP2_VERTEX_3);
      gl.glEnable(gl.GL_AUTO_NORMAL);
      gl.glEnable(gl.GL_NORMALIZE);

      gl.glMap2f(gl.GL_MAP2_VERTEX_3,
              0.0f,1.0f,3,munn.getUN(),
              0.0f,1.0f,3*munn.getUN(),munn.getVNleppe(),munn.getLeppe());
               
      gl.glEnable(GL_MAP2_VERTEX_3);
      
      gl.glMapGrid2f(40,0.0f,1.0f,40,0.0f, 1.0f); 
      gl.glFrontFace(gl.GL_CW);         
      gl.glEvalMesh2(fill_line, 0, 40, 0, 40);

      gl.glDisable(GL_TEXTURE_2D);
		gl.glPopMatrix(); //leppa
 		gl.glPopMatrix(); //Tennene
 		gl.glPopMatrix();	//Munnen


	}

	/**
	/* Metode som laster teksturene
	*/
	public void textures(String file) {
		 
		/** jeg har .png bilder derfor PngTextureLoader */  
   	PngTextureLoader txtLoader = new PngTextureLoader(gl, glu);
      txtLoader.readTexture(file);

      if(txtLoader.isOk()) {
         
         /** Genererer tekstur og legger den i arrayen */   
        	gl.glGenTextures(1,texName);  
         gl.glBindTexture(GL_TEXTURE_2D,texName[0]);
			
			/**
         *Her lager vi teksturen, gir hyde, bredde og annen informasjon om bildet
         */  
         gl.glTexImage2D(GL_TEXTURE_2D,
            0,
            3,
            txtLoader.getImageWidth(),
            txtLoader.getImageHeight(),
            0,
            GL_RGB,
            GL_UNSIGNED_BYTE,
            txtLoader.getTexture());

      }
      
    }

  }
}


